local select = select
local ipairs = ipairs
local GetTalentInfo = GetTalentInfo
local GRAY_FONT_COLOR = GRAY_FONT_COLOR
local NORMAL_FONT_COLOR = NORMAL_FONT_COLOR
local GREEN_FONT_COLOR = GREEN_FONT_COLOR
local RED_FONT_COLOR = RED_FONT_COLOR
local Talented = Talented

local L = AceLibrary("AceLocale-2.2"):new("Talented")

function Talented:UpdateCurrentTemplate()
	local info
	local template = self.current
	if not template then
		local class = select(2, UnitClass("player"))
		template = {
			name = "Current",
			class = class,
		}
		info = self:GetTalentInfo(class)
		for tab, tree in ipairs(info) do
			template[tab] = { talents = {} }
		end
		self.current = template
	else
		info = self:GetTalentInfo(template.class)
	end
	local total = UnitCharacterPoints("player") 
	for tab, tree in ipairs(info) do
		for index, info in ipairs(tree.talents) do
			local rank = select(5, GetTalentInfo(tab, index))
			template[tab][index] = rank
			total = total + rank
		end
	end
	self.maxpoints = total
	self.current = template
	if self.template == self.current then
		self:TriggerEvent("Talented_TalentChanged")
	end
end

function Talented:UpdateView()
	if not self.base then return end
	local template, target
	if self.mode == "apply" then
		template = self.current
	else
		template = self.template
	end
	local total = 0
		
	if template ~= self.current then
		self.base.editname:SetText(template.name)
	else
		self:MakeTarget()
		target = self.db.profile.target and self.target
	end

	local info = self:GetTalentInfo(template.class)
	if not info then return end
	local at_cap = self:IsTemplateAtCap(template)
	for tab, tree in ipairs(info) do
		local count = 0
		for index, talent in ipairs(tree.talents) do
			local rank = template[tab][index]
			if not rank then
				template[tab][index] = 0
				rank = 0
			end
			count = count + rank
			local button = talent.button
			local color = GRAY_FONT_COLOR
			local state = self:GetTalentState(template, tab, index)
			if state == "empty" and (at_cap or self.mode == "view") then
				state = "unavailable"
			end
			if state == "unavailable" then
				button.texture:SetDesaturated(1)
				button.slot:SetVertexColor(0.65, 0.65, 0.65)
				button.rank:Hide()
				button.rank.texture:Hide()
			else
				button.rank:Show()
				button.rank.texture:Show()
				button.rank:SetText(rank)
				button.texture:SetDesaturated(0)
				if state == "full" then
					color = NORMAL_FONT_COLOR
				else
					color = GREEN_FONT_COLOR
				end
				button.slot:SetVertexColor(color.r, color.g, color.b)
				button.rank:SetVertexColor(color.r, color.g, color.b)
			end
			if talent.info.prereqs then
				for _, req in ipairs(talent.info.prereqs) do
					local ecolor = color
					if ecolor == GREEN_FONT_COLOR then
						if self.mode == "edit" then
							local s = self:GetTalentState(template, tab, req.source)
							if s ~= "full" then
								ecolor = RED_FONT_COLOR
							end
						else
							ecolor = NORMAL_FONT_COLOR
						end
					end
					for _, element in ipairs(req.elements) do
						element:SetVertexColor(ecolor.r, ecolor.g, ecolor.b)
					end
				end
			end
			if target and target[tab][index] > 0 then
				local btarget = self:GetButtonTarget(button)
				btarget:Show()
				btarget.texture:Show()
				btarget:SetText(target[tab][index])
			elseif button.target then
				button.target:Hide()
				button.target.texture:Hide()
			end
		end
		tree.frame.name:SetText(L["%s (%d)"]:format(tree.info.name, count))
		total = total + count
		local clear = tree.frame.clear
		if self.mode ~= "edit" or count <= 0 or self.template == self.current then
			clear:Hide()
		else
			clear:Show()
		end
	end
	local points = self.base.points
	local maxpoints = self.maxpoints
	if self.db.profile.show_level_req then
		local level_req = (total == 0) and 1  or (9 + total)
		points:SetText(L["Level %d"]:format(level_req))
	else
		points:SetText(L["%d/%d"]:format(total, maxpoints))
	end
	local color
	if total < maxpoints then
		color = GREEN_FONT_COLOR
	elseif total > maxpoints then
		color = RED_FONT_COLOR
	else
		color = NORMAL_FONT_COLOR
	end
	points:SetTextColor(color.r, color.g, color.b)
end
